/*------------------------------------------------------------------------.
| Copyright (C) 2000 Alexandre Duret-Lutz <duret_g@epita.fr>              |
|                                                                         |
| This file is part of Heroes.                                            |
|                                                                         |
| Heroes is free software; you can redistribute it and/or modify it under |
| the terms of the GNU General Public License as published by the Free    |
| Software Foundation; either version 2 of the License, or (at your       |
| option) any later version.                                              |
|                                                                         |
| Heroes is distributed in the hope that it will be useful, but WITHOUT   |
| ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or   |
| FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License   |
| for more details.                                                       |
|                                                                         |
| You should have received a copy of the GNU General Public License along |
| with this program; if not, write to the Free Software Foundation, Inc., |
| 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA                   |
`------------------------------------------------------------------------*/

/* $Id: argv.c 1.4 Sun, 21 May 2000 12:13:02 +0200 pollux $ */

#include <stdio.h>
#include <getopt.h>
#include "config.h"
#include "const.h"
#include "argv.h"

int snap = 0;
int cpuon = 1;
int nosfx = 0;
int joyoff = 0;
int devparm = 0;
int loadulevel = 0;
int directmenu = 0;
int reinitsco = 0;
int reinitopt = 0;
int reinitsav = 0;
int x10sav = 0;
int doublefx = 1;
int swapside = 1;
char level_name[256];

static void
version ()
{
  puts ("Heroes " VERSION "\n");
  puts ("Copyright (c) 1997,1998,2000 "
	"Alexandre Duret-Lutz, Romuald Genevois,\n"
	"Alexandre Liverneaux and Philippe Meisburger.\n");
  puts ("This is free software; see the source for copying conditions.  "
	"There is NO\nwarranty; not even for MERCHANTABILITY or FITNESS"
	" FOR A PARTICULAR PURPOSE.");
}

static void
print_help (char* argv0)
{
  printf ("Usage: %s [OPTIONS]...\n\n",argv0);
  puts ("Heroes is a game like nibbles but different.\n\n"
	"  -v, --version\t\t  display version\n"
	"  -h, --help\t\t  display this help\n"
	"      --cpu-off\t\t  disable computer opponents\n"
	"      --default-scores\t  restore default scores file\n"
	"      --default-options\t  restore default options file\n"
	"      --default-saves\t  restore default saves file\n"
	"  -s, --swap-sides\t  swap sides in two player mode\n"
	"      --no-double-fx\t  disable superposition of rotozoom and waves\n"
	"  -g, --go\t\t  skip the introduction\n"
	"\n"
	"Report bugs to <duret_g@epita.fr>.");
}

const struct option long_options[] = {
  {"version",		0, 0,		'v'},
  {"help",		0, 0,		'h'},
  {"cpu-off",		0, &cpuon,	0},
  {"default-scores",	0, &reinitsco,	1},
  {"default-options",	0, &reinitopt,	1},
  {"default-saves",	0, &reinitsav,	1},
  {"x10-saves",		0, &x10sav,	1},
  {"devparm",		0, &devparm,	1},
  {"snap",		0, &snap,	0},
  {"joystick-off",	0, &joyoff,	1},
  {"swap-sides",	0, 0,		's'},
  {"no-sfx",		0, 0,		'X'},
  {"no-double-fx",	0, &doublefx,	0},
  {"load",		1, 0,		'l'},
  {"go",		0, 0,		'g'},
  {0,			0, 0,		0}
};

int
parse_argv (int argc, char **argv)
{
  int c;

  for (;;) {
    int option_index = 0;

    c = getopt_long (argc, argv, "vh2sXl:", long_options, &option_index);

    /* Detect the end of the options. */
    if (c == -1)
      break;

    switch (c) {
    case 'v':
      version ();
      return 1;
    case 'h':
      print_help (argv[0]);
      return 1;
    case 's':
      swapside = 0;
      break;
    case 'X':
      nosfx = 1;
      break;
    case 'l':
      strcpy (level_name, nivdir);
      strcat (level_name, optarg);
      strcat (level_name, ".lvl");
      loadulevel = 1;
      break;
    case 'g':
      directmenu = 1;
      break;
    case '?':
      /* getopt_long already printed an error message. */
    case 0:
      break;
      
    default:
      abort ();
    }
  }
  return 0;
}
