/*------------------------------------------------------------------------.
| Copyright (C) 1997,1998,2000 Alexandre Duret-Lutz <duret_g@epita.fr>    |
|                                                                         |
| This file is part of Heroes.                                            |
|                                                                         |
| Heroes is free software; you can redistribute it and/or modify it under |
| the terms of the GNU General Public License as published by the Free    |
| Software Foundation; either version 2 of the License, or (at your       |
| option) any later version.                                              |
|                                                                         |
| Heroes is distributed in the hope that it will be useful, but WITHOUT   |
| ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or   |
| FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License   |
| for more details.                                                       |
|                                                                         |
| You should have received a copy of the GNU General Public License along |
| with this program; if not, write to the Free Software Foundation, Inc., |
| 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA                   |
`------------------------------------------------------------------------*/

/* $Id: display.c 1.8 Wed, 24 May 2000 09:29:53 +0200 pollux $ */

#include <stdlib.h>
#include <stdio.h>
#include <ggi/ggi.h>
#include "display.h"

unsigned char *screen;

ggi_visual_t visu;
static ggi_mode vid_mode;

void
init_video (void)
{

  const ggi_directbuffer *db;

  if (ggiInit ())
    exit (EXIT_FAILURE);

  visu = ggiOpen (NULL);

  vid_mode.frames = 1;
  vid_mode.visible.x = 320;
  vid_mode.visible.y = 200;
  vid_mode.virt.x = vid_mode.virt.y = GGI_AUTO;
  vid_mode.dpp.x = vid_mode.dpp.y = GGI_AUTO;
  vid_mode.graphtype = GT_8BIT;

  if (ggiSetMode (visu, &vid_mode)) {
    fprintf (stderr, "Can't open a correct display.\n");
    /* try to use the suggested mode */
    // if (ggiSetMode (visu, &vid_mode))
    exit (EXIT_FAILURE);
  }


  db = ggiDBGetBuffer (visu, 0);
  if (!db || !(db->type & GGI_DB_SIMPLE_PLB)) {
    //    printf ("%p\n", db);
    fprintf (stderr, "Can't correct direct-buffer");
    exit (EXIT_FAILURE);
  }
  screen = db->write;

  ggiAddFlags (visu, GGIFLAG_ASYNC);
  ggiSetColorfulPalette (visu);
}

void
uninit_video (void)
{
  ggiClose (visu);
  ggiExit ();
}


void
set_color (unsigned char c, unsigned char r, unsigned char g, unsigned char b)
{
  ggi_color cmap[256];
  cmap[c].r = r * 1024;
  cmap[c].g = g * 1024;
  cmap[c].b = b * 1024;
  ggiSetPalette (visu, c, 1, cmap);
}

void
set_pal (unsigned char *ptr, int p, int n)
{
  signed i;

  ggi_color cmap[256];
  for (i = 0; i < 256; ++i) {
    cmap[i].r = *ptr++ * 1024;
    cmap[i].g = *ptr++ * 1024;
    cmap[i].b = *ptr++ * 1024;
  }
  ggiSetPalette (visu, p / 3, n / 3, cmap);
}

void
vsynchro (void)
{
  ggiFlush (visu);
}
