/*------------------------------------------------------------------------.
| Copyright (C) 1997,1998,2000 Alexandre Duret-Lutz <duret_g@epita.fr>    |
|                                                                         |
| This file is part of Heroes.                                            |
|                                                                         |
| Heroes is free software; you can redistribute it and/or modify it under |
| the terms of the GNU General Public License as published by the Free    |
| Software Foundation; either version 2 of the License, or (at your       |
| option) any later version.                                              |
|                                                                         |
| Heroes is distributed in the hope that it will be useful, but WITHOUT   |
| ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or   |
| FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License   |
| for more details.                                                       |
|                                                                         |
| You should have received a copy of the GNU General Public License along |
| with this program; if not, write to the Free Software Foundation, Inc., |
| 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA                   |
`------------------------------------------------------------------------*/

/* $Id: pcx.c 1.8 Mon, 22 May 2000 09:53:50 +0200 pollux $ */

#include <stdio.h>
#include <stdlib.h>
#include "errors.h"
#include "pcx.h"

static void
img_init (image_ * image)
{
  image->buffer = (char *) malloc (image->taille);
  if (image->buffer == NULL)
    fatal_error ("[PCX] Not enough memory.");
}

void
img_free (image_ * image)
{
  free ((char *) image->buffer);
}

static void
delta (image_ * image)
{
  int i;
  char *src = image->buffer + image->xt;
  for (i = image->taille - image->xt; i != 0; i--)
    (*src++) = (char) ((*src) + (*(src - image->xt)));
}

char
pcx_load (const char *fichier, image_ * image)
{
  long compteur;
  FILE *fptr;

  int nbrbytes, i;
  unsigned char data;

  if ((fptr = fopen (fichier, "rb")) == NULL) {
    puts (fichier);
#ifndef __HEDIT__
    fatal_error ("Unable to open this PCX\n");
#else
    fatalog ("Unable to open this PCX\n");
#endif
  }
  fread (&(image->entete), sizeof (entete_), 1, fptr);

  image->xt = (image->entete.largeur - image->entete.x + 1);
  image->yt = (image->entete.hauteur - image->entete.y + 1);
  image->taille = image->xt * image->yt;

  img_init (image);

  compteur = 0;
  if (image->entete.rle)
    while (compteur < image->taille) {
      data = (unsigned char) getc (fptr);
      if ((data & 192) == 192) {
	nbrbytes = data & 63;
	data = (unsigned char) getc (fptr);
	while (nbrbytes--)
	  image->buffer[compteur++] = data;
      } else
	image->buffer[compteur++] = data;
  } else
    fread (image->buffer, image->taille, 1, fptr);
  if (image->entete.rle == 2)
    delta (image);


  data = (unsigned char) getc (fptr);	// data==0Ch normallement

  fread (&(image->palette), 768, 1, fptr);
  for (i = 0; i < 256 * 3; i++)
    image->palette.global[i] =
      (unsigned char) (image->palette.global[i] >> 2);

  fclose (fptr);
  return (0);
}
