/*------------------------------------------------------------------------.
| Copyright (C) 1997,1998,2000 Alexandre Duret-Lutz <duret_g@epita.fr>    |
|                                                                         |
| This file is part of Heroes.                                            |
|                                                                         |
| Heroes is free software; you can redistribute it and/or modify it under |
| the terms of the GNU General Public License as published by the Free    |
| Software Foundation; either version 2 of the License, or (at your       |
| option) any later version.                                              |
|                                                                         |
| Heroes is distributed in the hope that it will be useful, but WITHOUT   |
| ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or   |
| FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License   |
| for more details.                                                       |
|                                                                         |
| You should have received a copy of the GNU General Public License along |
| with this program; if not, write to the Free Software Foundation, Inc., |
| 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA                   |
`------------------------------------------------------------------------*/

/* $Id: savegame.c 1.10 Mon, 22 May 2000 09:53:50 +0200 pollux $ */

/* sauvegardes... */

#include "stdio.h"
#include "structs.h"
#include "string.h"
#include "scores.h"
#include "assert.h"
#include "savegame.h"
#include "config.h"
#include "argv.h"

#define N_MAGICS 40
saved_game saverec[10];
unsigned char magics[N_MAGICS];

static void
set_magic (unsigned char i)
{
  assert (i < N_MAGICS);
  magics[i] = 1;
}

unsigned char
compute_magic (void)
{
  int i;
  memset (magics, 0, N_MAGICS * sizeof (unsigned char));
  for (i = 0; i < 10; i++)
    if (saverec[i].used)
      set_magic (saverec[i].magic);
  for (i = 0; i < /*20 */ 10; i++)
    set_magic (highs[0][i].magic);
  for (i = 1; magics[i] == 1; i++);
  assert (i > 0);
  assert (i < N_MAGICS /* plus de magics ?? */ );
  assert (magics[i] == 0);
  return (i);
}

signed char
find_magic (unsigned char m)
{
  int i;
  for (i = 0; i < 10; i++)
    if (highs[0][i].magic == m)
      return (i);
  return (-1);
}


static unsigned long int
check_save_records (void)
{
  unsigned long int crc = 0xa8c4d602;
  int i;
  char *src = (char *) saverec;
  for (i = 0; i < (10 * sizeof (saved_game) - 4); i += 3, src += 3)
    crc ^= *(int *) src;
  return (crc);
}

void
clear_save_records (void)
{
  memset (saverec, 0, 10 * sizeof (saved_game));
}

void
write_save_records (void)
{
  FILE *fs;
  unsigned int i;
  fs = fopen (confdir "savegame.dat", "wb");
  i = check_save_records ();
  fwrite (saverec, sizeof (saved_game), 10, fs);
  fwrite ((int *) &i, 4, 1, fs);
  fclose (fs);
}

void
load_save_records (void)
{
  FILE *fs;
  unsigned long int i;
  fs = fopen (confdir "savegame.dat", "rb");
  if (fs == NULL)
    clear_save_records ();
  else {
    fread (saverec, sizeof (saved_game), 10, fs);
    fread ((int *) &i, 4, 1, fs);
    fclose (fs);
    if (check_save_records () != i)
      clear_save_records ();
  }
  if (x10sav)
    for (i = 0; i < 10; i++) {
      sprintf (saverec[i].name, "LVL%ld", (1 + i) * 10);
      saverec[i].level = (1 + i) * 10 - 1;
      saverec[i].points[0] = saverec[i].points[1] = saverec[i].points[2] =
	saverec[i].points[3] = (1 + i) * 1000;
      saverec[i].lifes[0] = saverec[i].lifes[1] = saverec[i].lifes[2] =
	saverec[i].lifes[3] = 9;
      saverec[i].magic = 0;
      saverec[i].used = 1;
    }
}
