/*------------------------------------------------------------------------.
| Copyright (C) 1997,1998,2000 Alexandre Duret-Lutz <duret_g@epita.fr>    |
|                                                                         |
| This file is part of Heroes.                                            |
|                                                                         |
| Heroes is free software; you can redistribute it and/or modify it under |
| the terms of the GNU General Public License as published by the Free    |
| Software Foundation; either version 2 of the License, or (at your       |
| option) any later version.                                              |
|                                                                         |
| Heroes is distributed in the hope that it will be useful, but WITHOUT   |
| ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or   |
| FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License   |
| for more details.                                                       |
|                                                                         |
| You should have received a copy of the GNU General Public License along |
| with this program; if not, write to the Free Software Foundation, Inc., |
| 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA                   |
`------------------------------------------------------------------------*/

/* $Id: sfx.c 1.13 Sat, 20 May 2000 18:37:42 +0200 pollux $ */

#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <ctype.h>
#include <string.h>
#include "errors.h"
#include "options.h"
#include "sfx.h"
#include "config.h"

#ifdef PORT
  /* FIXME: SFX */

#define max_events 151

typedef char filename_t[13];

static filename_t *sfx_names;
static char *sfx_loaded;
static int *sfx_handles;
static int *play_handles;
static int event_handle[max_events];
static int max_sfx = 0;
static char tmp3[256];

static void
remove_comments (char *str)
{
  int i;
  for (i = 0;
       (str[i] != ';') && (str[i] != 0) && (str[i] != 13) && (str[i] != 10);
       i++);
  i--;
  for (; str[i] == ' '; i--);
  str[i + 1] = 0;
}

static char *
forward_spaces (char *tmp)
{
  while (*tmp == ' ')
    tmp++;
  return (tmp);
}
#endif

char
read_sfx_conf (void)
{
#ifdef PORT
  /* FIXME: SFX */
  FILE *fconf;
  char tmp[256];
  char *tmpptr = (char *) tmp;
  char *tmpptr2;
  char c;
  int nbr;

  if (nosfx)
    return (0);

  if ((fconf = fopen (sfxdir "sfx.cfg", "rt")) == NULL)
    return (-1);
  while (fgets (tmpptr, 256, fconf) != NULL) {
    c = toupper (tmp[0]);
    remove_comments (tmpptr);
    if (c == 'M') {
      if (sfx_names != NULL)
	return (-1);
      max_sfx = atol (tmpptr + 1);
      // this needs to be rewriten completly :
      sfx_names = malloc (max_sfx * sizeof (filename_t));
      sfx_loaded = malloc (max_sfx);
      sfx_handles = malloc (max_sfx * sizeof (int));
      play_handles = malloc (max_sfx * sizeof (int));
      memset (sfx_names, 0, max_sfx * sizeof (filename_t));
      memset (sfx_loaded, 0, max_sfx);
      memset (sfx_handles, 0, max_sfx * sizeof (int));
      memset (play_handles, 0, max_sfx * sizeof (int));
    } else if (c == 'F') {
      tmpptr2 = strchr (tmpptr, ' ');
      *tmpptr2++ = 0;
      nbr = atol (tmpptr + 1);
      tmpptr2 = forward_spaces (tmpptr2);
      if (nbr < max_sfx)
	strncpy ((*sfx_names)[nbr], tmpptr2, 13);
    } else if (c == 'E') {
      tmpptr2 = strchr (tmpptr, ' ');
      *tmpptr2++ = 0;
      nbr = atol (tmpptr + 1);
      tmpptr2 = forward_spaces (tmpptr2);
      if (nbr < max_events)
	event_handle[nbr] = atol (tmpptr2);
    }
  }
  fclose (fconf);
#endif
  return (0);
}

void
close_sfx_handle (void)
{
#ifdef PORT
  /* FIXME: SFX */
  if (nosfx)
    return;
  free (play_handles);
  free (sfx_handles);
  free (sfx_loaded);
  free (sfx_names);
#endif
}

#define mark_sfx(n) { assert(event_handle[(n)]<max_sfx); (*sfx_loaded)[event_handle[(n)]]=1;}

#ifdef PORT
  /* FIXME: SFX */
static void
std_sfx_set (void)
{
  int i;
  for (i = 20; i < 36; i++)
    mark_sfx (i);
  for (i = 37; i < 70; i++)
    mark_sfx (i);
  for (i = 85; i < 90; i++)
    mark_sfx (i);
  for (i = 120; i < 130; i++)
    mark_sfx (i);
  for (i = 140; i < 150; i++)
    mark_sfx (i);
}
#endif

void
free_all_sfx (void)
{
#ifdef PORT
  /* FIXME: SFX */
  int i;
  if (nosfx)
    return;
  for (i = 1; i < max_sfx; i++)
    if ((*sfx_loaded)[i]) {
      fxFreeSample ((*sfx_handles)[i]);
      (*sfx_loaded)[i] = 0;
    }
#endif
}

void
load_sfx_mode (signed char mode)
{
#ifdef PORT
  /* FIXME: SFX */
  int i;

  if (nosfx)
    return;

  free_all_sfx ();

/* reperage des sfx  charger */

  switch (mode) {
  case -1:			/* menus */
    for (i = 1; i < 20; i++)
      mark_sfx (i);
    for (i = 70; i < 80; i++)
      mark_sfx (i);
    for (i = 110; i < 120; i++)
      mark_sfx (i);
    for (i = 130; i < 140; i++)
      mark_sfx (i);
    break;
  /*quest */ case 0:
    std_sfx_set ();
    break;
  /*death */ case 1:
    std_sfx_set ();
    break;
  /*kilem */ case 2:
    for (i = 90; i < 100; i++)
      mark_sfx (i);
    std_sfx_set ();
    break;
  /*tca$h */ case 3:
    mark_sfx (36);
    mark_sfx (80);
    mark_sfx (81);
    std_sfx_set ();
    break;
  /*color */ case 4:
    for (i = 100; i < 106; i++)
      mark_sfx (i);
    std_sfx_set ();
    break;
  default:
    assert (0 /* sfx-mode inconnu ! */ );
    break;
  }

/* chargement des sfx marqus */

  for (i = 1; i < max_sfx; i++)
    if ((*sfx_loaded)[i]) {
      strcat (strcpy ((char *) tmp3, sfxdir), (char *) &((*sfx_names)[i]));
      if (fxLoadRawSample
	  ((char *) tmp3, smp8bitMono, sdLoopNone,
	   (unsigned *) &((*sfx_handles)[i])))
	fatal_error ("chargement sample impossible :-(");
    }
#endif
}


void
event_sfx (int event)
{
#ifdef PORT
  /* FIXME: SFX */
  static int channelsfx = 0;
  int i = (*sfx_handles)[event_handle[event]];
  if (nosfx)
    return;
  assert (event < max_events);
  assert (event_handle[event] < max_sfx);
  if (event_handle[event] != 0) {
    assert ((*sfx_loaded)[event_handle[event]]);
    fxPlaySample (channelsfx, i, 1, 11025, (13 - opt.sfx_volume) * 64 / 13, 0,
		  (unsigned *) &((*play_handles)[event_handle[event]]));
    channelsfx = (channelsfx + 1) & 3;
  }
#endif
}
