/*------------------------------------------------------------------------.
| Copyright (C) 1997,1998,2000 Alexandre Duret-Lutz <duret_g@epita.fr>    |
|                                                                         |
| This file is part of Heroes.                                            |
|                                                                         |
| Heroes is free software; you can redistribute it and/or modify it under |
| the terms of the GNU General Public License as published by the Free    |
| Software Foundation; either version 2 of the License, or (at your       |
| option) any later version.                                              |
|                                                                         |
| Heroes is distributed in the hope that it will be useful, but WITHOUT   |
| ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or   |
| FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License   |
| for more details.                                                       |
|                                                                         |
| You should have received a copy of the GNU General Public License along |
| with this program; if not, write to the Free Software Foundation, Inc., |
| 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA                   |
`------------------------------------------------------------------------*/

/* $Id: sound.c 1.9 Sat, 20 May 2000 18:04:05 +0200 pollux $ */

#include <stdlib.h>
#include "sound.h"
#include "config.h"

char setup = 0;
int error;
unsigned int sync;
char mono = 0;


#ifdef HAVE_LIBMIKMOD
#include <mikmod.h>
#include <pthread.h>
#include <signal.h>
#include <errno.h>
#include "options.h"

MODULE *module;
pthread_t polling_thread;
pthread_mutex_t playing;	/* this mutex is used to tell the polling 
				   thread that it must continue polling ... */

void
set_volume (void)
{
#ifdef PORT
  /* FIXME: SOUND */
  if (opt.music)
    midasSD->SetMasterVolume ((13 - opt.music_volume) * 64 / 13);
  else
    midasSD->SetMasterVolume (0);
#endif
}

void
halve_volume (void)
{
#ifdef PORT
  /* FIXME: SOUND */
  int j;
  midasSD->GetMasterVolume ((unsigned *) &j);
  midasSD->SetMasterVolume (j / 2);
#endif
}

int
init_sound_engine (void)
{
  /* register all the drivers */
  MikMod_RegisterAllDrivers ();

  /* register the xm module loader */
  MikMod_RegisterLoader (&load_xm);

  /* initialize the library */
  md_mode |= DMODE_SOFT_MUSIC;
  if (MikMod_Init ("")) {
    fprintf (stderr, "Could not initialize sound, reason: %s\n",
	     MikMod_strerror (MikMod_errno));
    return 1;
  }

  if (MikMod_InitThreads () != 1) {
    fprintf (stderr, "libmikmod is not thread safe\n");
    return 1;
  }

  pthread_mutex_init (&playing, 0);
  set_volume ();

#ifdef PORT
  /* FIXME: SFX */
  if (opt.sfx)
    fxInit (midasSD);
#endif
  return 0;
}

void
uninit_sound_engine (void)
{
  MikMod_Exit ();
}

void
load_soundtrack (char *ptr)
{
  module = Player_Load (ptr, 16, 0);
}

void
unload_soundtrack (void)
{
  pthread_mutex_unlock (&playing);
  pthread_join (polling_thread, 0);
  Player_Stop ();
  Player_Free (module);
}

static void *
update_thread (void *arg)
{
  while (pthread_mutex_trylock (&playing) == EBUSY) {
    MikMod_Update ();
    usleep (10000);
  }
  pthread_mutex_unlock (&playing);
  return 0;
}

void
play_soundtrack (void)
{
  pthread_mutex_lock (&playing);
  Player_Start (module);
  pthread_create (&polling_thread, 0, update_thread, 0);
}


#else // not HAVE_LIBMIKMOD

/* empty implementation */

void
set_volume (void)
{
}

void
halve_volume (void)
{
}

int
init_sound_engine (void)
{
  return 0;
}

void
uninit_sound_engine (void)
{
}

void
load_soundtrack (char *ptr)
{
}

void
unload_soundtrack (void)
{
}

void
play_soundtrack (void)
{
}

#endif
